define([
    "jquery", "underscore", "backbone",
    "collections/snippets", "collections/my-form-snippets",
    "views/tab", "views/my-form"
], function(
    $, _, Backbone, SnippetsCollection, MyFormSnippetsCollection, TabView, MyFormView
) {
    return {
        initialize: function() {
            // merge all templates
            _.extend(_snippetTemplates, _actionTemplates);
            //Bootstrap tabs from json.
            if (_.size(_snippetData['input'])) {
                new TabView({
                    title: "Input",
                    collection: new SnippetsCollection(_snippetData['input'])
                });
            }
            if (_.size(_snippetData['radio'])) {
                new TabView({
                    title: "Radios / Checkboxes",
                    collection: new SnippetsCollection(_snippetData['radio'])
                });
            }
            if (_.size(_snippetData['select'])) {
                new TabView({
                    title: "Select",
                    collection: new SnippetsCollection(_snippetData['select'])
                });
            }
            if (_.size(_snippetData['button'])) {
                new TabView({
                    title: "Buttons",
                    collection: new SnippetsCollection(_snippetData['button'])
                });
            }
            if (_.size(_actionData['action'])) {
                new TabView({
                    title: "Actions",
                    collection: new SnippetsCollection(_actionData['action'])
                });
            }

            //Make the first tab active!
            $('#formFactoryComponentsTabs').find('a:first').tab('show');

            activateAutoNavInTab();

            addButtonsToForm(formJson);

            // Bootstrap "My Form" with 'Form Name' snippet.
            formView = new MyFormView({
                title: "Original",
                collection: new MyFormSnippetsCollection(stepsTable[currentStep].stepContent)
            });
        }
    }
});

function addButtonsToForm(form) {
    if (form.formStepsNumber == 1) {
        var needNewButton = true;
        var needToRemoveTripleButton = false;
        _.each(form.formSteps[currentStep].stepContent, function(snippet){
            if (snippet.nodeType.match("fcnt:buttonTripleDefinition") || snippet.nodeType.match("fcnt:buttonDefinition")) {
                if (snippet.nodeType == "fcnt:buttonDefinition") {
                    if (snippet.fields.buttoneventid.value != "_eventId_submit") {
                        snippet.fields.buttoneventid.value = "_eventId_submit";
                        snippet.fields.buttonlabel.value = jsVarMap.labelSubmit;
                    }
                    needNewButton = false;
                } else {
                    needToRemoveTripleButton = true;
                }
            }
        });
        if (needToRemoveTripleButton) {
            form.formSteps[currentStep].stepContent = _.without(form.formSteps[currentStep].stepContent, _.last(form.formSteps[currentStep].stepContent));
        }
        if (needNewButton) {
            var singleButton = [];
            _.each(_snippetData.hiddenbutton, function(button){
                if(button.nodeType == "fcnt:buttonDefinition") {
                    singleButton = button;
                }
            });
            form.formSteps[currentStep].stepContent.push(singleButton);
        }
    } else {
        _.each(form.formSteps, function(step) {
            var needNewButton = true;
            var needToRemoveTripleButton = false;
            _.each(step.stepContent, function(snippet){
                if (snippet.nodeType.match("fcnt:buttonTripleDefinition") || snippet.nodeType.match("fcnt:buttonDefinition")){
                    var idNumber = step.stepNumber;
                    var stepPOS = form.formStepsNumber - 1;
                    if (idNumber != 0 && idNumber != stepPOS) {
                        if (snippet.fields.button3eventid.value != "_eventId_next") {
                            snippet.fields.button3eventid.value = "_eventId_next";
                            snippet.fields.button3label.value = jsVarMap.labelNext;
                        }
                        updateTripleButtonId(snippet, idNumber)
                    }
                    if (idNumber == stepPOS) {
                        if (snippet.fields.button3eventid.value != "_eventId_submit") {
                            snippet.fields.button3eventid.value = "_eventId_submit";
                            snippet.fields.button3label.value = jsVarMap.labelSubmit;
                        }
                        updateTripleButtonId(snippet, idNumber)
                    }
                    if (idNumber == 0) {
                        if (snippet.nodeType == "fcnt:buttonDefinition") {
                            if (snippet.fields.buttoneventid.value != "_eventId_next") {
                                snippet.fields.buttoneventid.value = "_eventId_next";
                                snippet.fields.buttonlabel.value = jsVarMap.labelNext;
                            }
                        } else {
                            needToRemoveTripleButton = true;
                        }
                    }
                    needNewButton = false;
                }
            });
            if (needToRemoveTripleButton) {
                form.formSteps[step.stepNumber].stepContent = _.without(form.formSteps[step.stepNumber].stepContent, _.last(form.formSteps[step.stepNumber].stepContent));
                var singleButton = [];
                _.each(_snippetData.hiddenbutton, function(button){
                    if(button.nodeType == "fcnt:buttonDefinition") {
                        singleButton = button;
                        singleButton.fields.buttoneventid.value = "_eventId_next";
                        singleButton.fields.buttonlabel.value = jsVarMap.labelNext;
                    }
                });
                form.formSteps[step.stepNumber].stepContent.push(singleButton);
            }
            if (needNewButton) {
                var tripleButton = [];
                _.each(_snippetData.hiddenbutton, function(button){
                    if(button.nodeType == "fcnt:buttonTripleDefinition") {
                        tripleButton = button;
                        var idNumber = (form.formStepsNumber - 1);
                        updateTripleButtonId(tripleButton, idNumber)
                    }
                });
                form.formSteps[step.stepNumber].stepContent.push(tripleButton);
            }
        });
    }
}

function updateTripleButtonId(snippet, idNumber) {
    snippet.fields.id.value = "triplebutton_" + idNumber;
    snippet.fields.button1_id.value = "button1_id_" + idNumber;
    snippet.fields.button2_id.value = "button2_id_" + idNumber;
    snippet.fields.button3_id.value = "button3_id_" + idNumber;
}

function activateAutoNavInTab() {
    $('#formFactoryComponentsTabs').find('a[data-toggle="tab"]').on('show', function (e) {
        var $stepActionLi = $("#stepActionLi");
        if ($(e.target).attr("href") == "#actions") {
            if (!$stepActionLi.hasClass("active")) {
                $stepActionLi.find("a").trigger("click");
            }
        } else {
            if ($stepActionLi.hasClass("active")) {
                $("#step0").find("a").trigger("click");
            }
        }
    });
}