//Underscore initialization parameters
_.templateSettings = {
    interpolate: /\<\@\=(.+?)\@\>/gim,
    evaluate: /\<\@(.+?)\@\>/gim,
    escape: /\<\@\-(.+?)\@\>/gim
};

//Backbone definition - Start

//Metadata View

//Define the view with a template displaying all results in a table body
//This model will be called with a Collection of results model
// so it has to browse the models one by one to display them
var MetadataResultsView = Backbone.View.extend({
    initialize: function(options) {
        //Initialize view
        this.collection = options.collection;
        this.oDataTable = options.oDataTable;
        this.$tableEl = options.$tableEl;

        //fill dataTable with rows from collection
        if (this.collection.length > 0) {
            var rows = this.collection.getRows(labels, this.collection.site, true);
            this.oDataTable.rows.add(rows).draw(false);
        }
    }
});


//Results Page View

//Define the view with a template displaying all results in a table body
//This model will be called with a Collection of results model
// so it has to browse the models one by one to display them
var FormsResultsView = Backbone.View.extend({
    displayFormTemplate: _.template("<@ if(actions.length > 0){ @>" +
        "   <form class='selectViewForm form-inline' name='select-view-from' id='<@= resultFormId @>'>" +
        "       <input type='hidden' value='<@= resultFormId @>' name='formUuid' class='form-input' id='formInput'>" +
        "       <input type='hidden' value='' name='actionview' class='form-input' id='actionview'>" +
        "       <input type='hidden' value='' name='actionUUID' class='form-input' id='actionInput'>" +
        "       <select name='view' style='vertical-align: top'>" +
        "       <@ _.each(actions, function(action){ @>" +
        "           <@ var actionData = _.find(actionsProviders, function(actionProvider){ return actionProvider.name == action }); @>" +
        "           <@ if (!_.isUndefined(actionData)) { @>" +
        "               <@ _.each(_.find(actionsViews, function(actionViews){return actionViews.actionName == action}).views,function(view){ @>" +
        "                   <option value='<@= view.viewName @>' actionId='<@= actionData.id @>'><@= view.viewLabel @></option>" +
        "            <@ }); @>" +
        "           <@ } @>" +
        "       <@ }); @>" +
        "       </select>" +
        "       <button class='btn btn-primary' type='submit' onclick='submitForm(\"<@= resultFormId @>\")'><@= buttonText @></button>" +
        "   </form>" +
        "<@ } @>"),
    initialize: function() {
        //Set the view htmlTag on the body tag of the result table
        this.$tableEl = $('#select_form');

        //Listen for the collection change to update the render
        this.listenTo(this.collection, 'sync', this.renderData);
    },
    renderData: function() {
        if (!this.resultToDisplay) {
            var columns = [
                {"title": this.nameHeader, "data": "formName"},
                {"title": this.dateHeader, "data": "created"},
                {"title": this.subCountHeader,"data":"subCount", "width":"10%"},
                {"title": this.displayHeader, "data": "displayForm"}
            ];
            //formName, created, displayForm
            var dataTableNoResult = this.dataTableNoResult;
            var searchLabel = this.searchLabel;
            var resultsFromLabel = this.resultsFromLabel;
            var resultsToLabel = this.resultsToLabel;
            if (_.isUndefined(this.dataTable)) {

                this.dataTable = $('#select_form').DataTable({
                    dom: "<'row-fluid'><'row-fluid'f><'row-fluid'rt><'row-fluid'i>",
                    columns: columns,
                    paging: false,
                    deferRender: true,
                    ordering: false,
                    language: {
                        search: searchLabel + "&nbsp;&nbsp;",
                        info: resultsFromLabel + " _START_ " + resultsToLabel + " _END_",
                        emptyTable: dataTableNoResult
                    },
                    createdRow: function(row, data, index) {
                        var that = $('td', row).eq(2);
                        var copyData = data;
                        if (data.entryPoints.length > 0) {
                            $.get(CONTEXT + data.entryPoints[0].entryPoint + "/total/" + data.formId, null, function(results) {
                                $(that).html(results.data);
                                copyData.resultsCount = results.data;
                            });
                        }
                    }
                });
            }
            var formTemplate = this.displayFormTemplate;
            var buttonText = this.buttonText;
            var dataTableRows = _.map(this.collection.models, function(model) {
                var datatableRow = model.toJSON();
                datatableRow.created = moment(datatableRow.created).format('DD/MM/YYYY h:mm:ss a');
                datatableRow.buttonText = buttonText;
                datatableRow.displayForm = formTemplate(datatableRow);
                datatableRow.subCount = "N/A";
                return datatableRow;
            });

            // Add rows in datatable if needed
            this.dataTable.rows.add(dataTableRows).draw();
        }
    }
});


PageableResultsView = Backbone.View.extend({
    initialize: function(options) {
        this.grid_el = options.grid_el;
        this.collection = options.collection;
        this.listenTo(this.collection, 'sync', this.render);
        getLabelsFromResults(labels, this.collection.formId);
        var columns = [];
        //Displaying only 5 first columns
        _.each(labels, function (labelObject, index) {
            var columnTitle = (labelObject.label)?labelObject.label:labelObject.htmlId;
            if (index < 5) {
                columns.push({"title": columnTitle});
            } else {
                columns.push({"title": columnTitle, "visible": false});
            }
        });

        this.datatable = this.grid_el.DataTable({
            dom: "<'row-fluid'T><'row-fluid'f><'row-fluid'rt><'row-fluid'i>S",
            columns: columns,
            scrollY: "400px",
            tableTools: {
                sSwfPath: options.swfUrl,
                aButtons: [
                    {
                        sExtends: "copy",
                        sButtonText: options.copyLabel
                    },
                    {
                        sExtends: "print",
                        sButtonText: options.printLabel,
                        sToolTip: options.printToolTipText,
                        sInfo: options.printInfoText
                    },
                    {
                        sExtends: "csv",
                        sButtonText: options.exportLabel + " CSV",
                        sAction: "flash_save"
                    }
                ]
            },
            language: {
                search: options.searchLabel + "&nbsp;&nbsp;",
                info: options.resultsFromLabel + " _START_ " + options.resultsToLabel + " _END_",
                emptyTable: options.noResultLabel
            },
            deferRender: true
        });
    },
    render: function() {
        if (this.collection.resetted) {
            this.datatable.clear().draw();
            this.collection.resetted = false;
        }
        if (this.collection.length > 0) {
            var rows = this.collection.getRows(labels, this.collection.site);
            this.datatable.rows.add(rows).draw(false);
            // Add checkbox in picker to select DataTable's column
            if (!this.pickers) {
                addCheckboxesInPicker(labels);
                this.pickers = true;
            }
        }
    }
});


/**
 * Render the Time submission view through a Line graph using chart.js
 */
TimeSubmissionView = Backbone.View.extend({
    initialize: function(options) {
        //Canvas element
        this.lineGraphEl = options.lineGraphEl;
        //Request parameters
        this.parameters = options.parameters;
        this.legends = options.legends;
        //Render graph
        this.renderData();
        //Listen for model changes
        this.listenTo(this.model, 'change', this.updateParameters);
    },
    url: function() {
        //Get url string from fetchURL function
        var url = getFetchURL({
            formId: this.model.formId,
            apiMethodName: "results_submissiontime",
            fromDate: this.model.fromDate,
            toDate: this.model.toDate
        });
        //Add parameters to url
        _.each(this.parameters, function(parameter,index) {
            var parameterString = "";
            var delimiter = index==0?"?":"&";
            var name = parameter.name;
            _.each(parameter.values, function(value,index) {
                var paramDelimiter = index>0?("&"):("");
                parameterString += paramDelimiter + name + "=" + value;
            });
            url += delimiter + parameterString;
        });
        return url;
    },
    renderData: function() {
        //Display patterns depending on servers patterns
        var patternMap = this.model.patternMap;

        //Getting the legend label
        var groupParam = _.find(this.parameters, function(parameter) {
            return parameter.name == "group_by";
        });
        var legendName = groupParam.values[0];

        var legend = _.find(this.legends, function(legend) {
            return legend.name == legendName;
        });
        //Chart.js line graph data
        var data = {
            graphData:{
                datasets: [{
                    label: legend.value,
                    fillColor: "rgba(220,220,220,0.2)",
                    strokeColor: "rgba(220,220,220,1)",
                    pointColor: "rgba(220,220,220,1)",
                    pointStrokeColor: "#fff",
                    pointHighlightFill: "#fff",
                    pointHighlightStroke: "rgba(220,220,220,1)",
                    data: []
                }],
                labels: []
            },
            max: 0
        };
        var tempLabels = [];
        var that = this;
        //Request API for graph data
        $.ajax({
            url : this.url(),
            type : 'GET',
            dataType : 'json',
            success: function(result) {
                var timeTotal = result.data.timeTotal;
                //Getting server time format pattern
                var serverPattern = result.data.groupingPattern;
                //Getting moment.js time format corresponding to server format
                var displayPattern = _.find(patternMap, function(patternMap) {
                    return patternMap.serverPattern == serverPattern;
                });
                //getting graph labels from results
                _.each(timeTotal, function(totalObject) {
                    var time = totalObject.timeKey;
                    if (tempLabels.indexOf(time) == -1) {
                        tempLabels.push(time);
                    }
                });
                //Sort the labels
                tempLabels.sort();
                _.each(tempLabels, function(label) {
                    var timeObject = _.find(timeTotal, function(totalObject) {return totalObject.timeKey == label});
                    if (timeObject != null) {
                        var total = timeObject.count;
                        if(total > data.max){
                            data.max = total;
                        }
                        data.graphData.datasets[0].data.push(total);
                    } else {
                        data.graphData.datasets[0].data.push(0);
                    }
                    data.graphData.labels.push(moment(label,serverPattern.toUpperCase()).format(displayPattern.displayPattern));
                });
                //Draw chart.js graph
                that.drawLineGraph(data);
            }
        });
    },
    drawLineGraph: function(data) {
        //Chart js line graph options
        var lineOptions = {
            scaleOverride: true,
            scaleStartValue: 0,
            scaleSteps: this.model.steps,
            scaleStepWidth: Math.ceil(data.max / this.model.steps)
        };
        //Get charts canvas
        var lineCanvas = this.lineGraphEl.find("canvas");

        //Get Chart.js needed context
        if (_.isUndefined(this.lineCtx))
            this.lineCtx = lineCanvas.get(0).getContext("2d");

        //Draw line chart if more than one point
        if(data.graphData.labels.length > 1){
            if (_.isUndefined(this.lineChart))
                this.lineChart = new Chart(this.lineCtx);

            var currentChart = this.lineChart.Line(data.graphData, lineOptions);
            this.lineGraphEl.find(".chart_legend").html(currentChart.generateLegend());
            this.lineGraphEl.find(".message").hide();
            lineCanvas.show();
            this.lineGraphEl.find(".chart_legend").show();
        } else {
            lineCanvas.hide();
            this.lineGraphEl.find(".chart_legend").hide();
            this.lineGraphEl.find(".message").show();
        }
    },
    updateParameters:function(){
        //Removing old parameters
        var fromDate = _.find(this.parameters, function(parameter) {
            return parameter.name=="fromDate";
        });
        var toDate = _.find(this.parameters, function(parameter) {
            return parameter.name=="toDate";
        });
        this.parameters = _.without(this.parameters,fromDate);
        this.parameters = _.without(this.parameters,toDate);

        //Pushing new values
        if (this.model.fromDate != null) {
            this.parameters.push({name:"fromDate",values:[this.model.fromDate]});
        }
        if (this.model.toDate != null) {
            this.parameters.push({name:"toDate",values:[this.model.toDate]});
        }
        //Render graph
        this.renderData();
    }
});

/**
 * Render the Page submission view through a Bar graph using chart.js
 */
PageSubmissionView = Backbone.View.extend({
    initialize: function(options) {
        //Canvas element
        this.barGraphEl = options.barGraphEl;
        //Request parameter
        this.parameters = options.parameters;
        //Render Bar graph
        this.renderData();
        //Listen for model changes
        this.listenTo(this.model, 'change', this.updateParameters);
    },
    url: function() {
        //Get url from Fetch URL function
        var url = getFetchURL({
            formId: this.model.formId,
            apiMethodName: "results_submissionpage",
            fromDate: this.model.fromDate,
            toDate: this.model.toDate
        });
        //Add request parameters to URL
        _.each(this.parameters, function(parameter,index) {
            var delimiter = (index == 0)?"?":"&";
            var parameterString = "";
            var name = parameter.name;
            _.each(parameter.values, function(value,index) {
                var paramDelimiter = (index > 0)?("&"):("");
                parameterString += paramDelimiter + name + "=" + value;
            });
            url += delimiter + parameterString;
        });
        return url;
    },
    renderData: function() {
        var patternMap = this.model.patternMap;
        //Graph Data
        var chartData = {graphData: {datasets:[], labels:[]}, max: 0};
        var tempLabels = [];
        var colors = ["220,220,220", "151,187,205", "247,70,74", "70,191,189", "253,180,92", "148,159,177", "77,83,96"];
        var colorIndex = 0;
        var that = this;
        //Get graph data from API
        $.ajax({
            url: this.url(),
            type: 'GET',
            dataType: 'json',
            success: function(result) {
                //Getting server time format pattern
                var serverPattern = result.data.groupingPattern;
                //Getting moment.js time format corresponding to server format
                var displayPattern = _.find(patternMap, function(patternMap) {
                    return patternMap.serverPattern == serverPattern;
                });

                //getting graph labels from results
                _.each (result.data.groupedCount, function(originObject) {
                    _.each(originObject.counts, function(count) {
                        if (tempLabels.indexOf(count.timekey) == -1) {
                            tempLabels.push(count.timekey);
                        }
                    });
                });
                //Sort the labels
                tempLabels.sort();
                //Getting results by ordered label
                _.each (result.data.groupedCount, function(originObject) {
                    var color = colors[(colors.length + colorIndex) % colors.length];
                    var data = [];
                    _.each(tempLabels, function(label) {
                        var count = _.find(originObject.counts, function(count) {return count.timekey == label});
                        if (count != null) {
                            if (count.count > chartData.max) {
                                chartData.max = count.count;
                            }
                            data.push(count.count);
                        } else {
                            data.push(0);
                        }
                    });
                    var currentDatataset = {
                        label: originObject.page,
                        fillColor: "rgba("+color+",0.5)",
                        strokeColor: "rgba("+color+",0.8)",
                        pointColor: "rgba("+color+",1)",
                        pointStrokeColor: "#fff",
                        pointHighlightFill: "#fff",
                        pointHighlightStroke: "rgba("+color+",1)",
                        data:data
                    };
                    chartData.graphData.datasets.push(currentDatataset);
                    colorIndex++;
                });
                //Formatting labels with moment.js
                _.each(tempLabels, function(label) {
                    chartData.graphData.labels.push(moment(label,serverPattern.toUpperCase()).format(displayPattern.displayPattern));
                });
                //Drawing Chart.js Bar graph
                that.drawBarGraph(chartData);
            }
        });
    },
    drawBarGraph: function(data) {
        var barCanvas = this.barGraphEl.find("canvas");

        var graphData = data.graphData;
        var barOptions = {
            scaleOverride: true,
            scaleStartValue: 0,
            scaleSteps: this.model.steps,
            scaleStepWidth: Math.ceil(data.max / this.model.steps)
        };

        //Get Chart.js needed context
        if (_.isUndefined(this.barCtx))
            this.barCtx = barCanvas.get(0).getContext("2d");

        //Draw bar chart if at least one bar
        if (graphData.labels.length > 0) {
            //Draw charts
            if (_.isUndefined(this.barChart))
                this.barChart = new Chart(this.barCtx);

            var currentChart = this.barChart.Bar(graphData, barOptions);
            this.barGraphEl.find(".chart_legend").html(currentChart.generateLegend());
            this.barGraphEl.find(".message").hide();
            barCanvas.show();
            this.barGraphEl.find(".chart_legend").show();
        } else {
            barCanvas.hide();
            this.barGraphEl.find(".chart_legend").hide();
            this.barGraphEl.find(".message").show();
        }
    },
    updateParameters: function() {
        //Removing old parameters
        var fromDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "fromDate";
        });
        var toDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "toDate";
        });
        this.parameters = _.without(this.parameters,fromDate);
        this.parameters = _.without(this.parameters,toDate);

        //Pushing new values
        if (this.model.fromDate != null) {
            this.parameters.push({name: "fromDate", values: [this.model.fromDate]});
        }
        if (this.model.toDate != null) {
            this.parameters.push({name: "toDate", values: [this.model.toDate]});
        }
        //Render graph
        this.renderData();
    }
});

/**
 * Render the Empty submission percentage view through a Table graph using Jquery datatable
 */
EmptySubmissionView = Backbone.View.extend({
    initialize: function(options) {
        this.tableEl = options.tableEl;
        this.parameters = options.parameters;
        this.totalValue = options.totalValue;
        this.datatableTitles = options.datatableTitles;
        this.noResultLabel = options.noResultLabel;
        this.renderData();
        //Listen for model changes
        this.listenTo(this.model, 'change', this.updateParameters);
    },
    url: function() {
        var url = getFetchURL({
            formId: this.model.formId,
            apiMethodName: "results_submissionempty",
            fromDate: this.model.fromDate,
            toDate: this.model.toDate
        });
        _.each(this.parameters, function(parameter,index) {
            var delimiter = (index == 0)?"?":"&";
            var parameterString = "";
            var name = parameter.name;
            _.each(parameter.values, function(value,index) {
                var paramDelimiter = (index > 0)?"&":"";
                parameterString += paramDelimiter + name + "=" + value;
            });
            url += delimiter + parameterString;
        });
        return url;
    },
    renderData: function() {
        var tableData = {rows: [], total: 0};
        var rows = [];
        var that = this;
        $.ajax({
            url: this.url(),
            type: 'GET',
            dataType: 'json',
            success: function(result) {
                tableData.total = result.data.totalCount;
                if (tableData.total > 0) {
                    _.each(result.data.countByField, function(countObject) {
                        var percentage = (tableData.total - countObject.count) * 100 / tableData.total;
                        tableData.rows.push([countObject.field, Math.round(percentage) + "%"]);
                    });
                }
                that.drawTable(tableData);
            }
        });
    },
    drawTable: function(tableData) {
        //Create or clear the datatable
        if (_.isUndefined(this.datatable)) {
            var columns = [];
            _.each(this.datatableTitles, function(title) {
                columns.push({title: title});
            });
            var noResultLabel = this.noResultLabel;
            this.datatable = this.tableEl.DataTable({
                dom: "rt",
                columns: columns,
                scrollY: "auto",
                paging: false,
                deferRender: true,
                info: false,
                language: {
                  emptyTable: noResultLabel
                },
                searching: false
            });
        } else {
            this.datatable.clear();
        }
        //add row in datatable if needed
        if (tableData.total > 0) {
            this.datatable.rows.add(tableData.rows);
        }
        //Redraw the datatable
        this.datatable.draw();
    },
    updateParameters: function() {
        //Removing old parameters
        var fromDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "fromDate";
        });
        var toDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "toDate";
        });
        this.parameters = _.without(this.parameters, fromDate);
        this.parameters = _.without(this.parameters, toDate);

        //Pushing new values
        if (this.model.fromDate != null) {
            this.parameters.push({name: "fromDate", values: [this.model.fromDate]});
        }
        if (this.model.toDate != null) {
            this.parameters.push({name: "toDate", values: [this.model.toDate]});
        }
        //Render graph
        this.renderData();
    }
});

/**
 * Render the choice fields view through a Pie graph using chart.js
 */
ChoiceFieldView = Backbone.View.extend({
    initialize: function(options) {
        this.pieGraphEl = options.pieGraphEl;
        this.parameters = options.parameters;
        var url = getFetchURL({
            formId: this.model.formId,
            apiMethodName: "results_choicelabel",
            fromDate: this.model.fromDate,
            toDate: this.model.toDate
        });
        var that = this;
        $.ajax({
            url: url,
            type: 'GET',
            dataType: 'json',
            success: function(result) {
                var choiceFields = result.data.choiceFields;
                //Render the choice fields selection and graph
                var choiceSelectEl = that.pieGraphEl.find("select");
                choiceSelectEl.html("");
                _.each(choiceFields, function(choiceFieldObject) {
                    if (_.isUndefined(that.selectChoice))
                        that.selectChoice = choiceFieldObject.id;
                    choiceSelectEl.append("<option value='" + choiceFieldObject.id + "'>" + choiceFieldObject.label + "</option>");
                });
                if (!_.isUndefined(that.selectChoice)) {
                    //Show field selection form
                    that.pieGraphEl.find("form").hide();
                    that.pieGraphEl.find("form").show();
                    //Draw chart for select first value
                    that.renderData();
                } else {
                    that.pieGraphEl.find(".message").show();
                    that.pieGraphEl.find("canvas").hide();
                    that.pieGraphEl.find(".chart_legend").hide();
                }
                choiceSelectEl.on("change", function(e) {
                    that.selectChoice = $(e.currentTarget).val();
                    that.renderData();
                });
            }
        });

        //Listen for model changes
        this.listenTo(this.model, 'change', this.updateParameters);
    },
    url: function() {
        var url = getFetchURL({
            formId: this.model.formId,
            apiMethodName: "results_choice",
            fromDate: this.model.fromDate,
            toDate: this.model.toDate,
            choiceId: this.selectChoice
        });
        _.each(this.parameters, function(parameter, index) {
            var delimiter = (index == 0)?"?":"&";
            var parameterString = "";
            var name = parameter.name;
            _.each(parameter.values, function(value, index) {
                var paramDelimiter = (index > 0)?"&":"";
                parameterString += paramDelimiter + name + "=" + value;
            });
            url += delimiter+parameterString;
        });
        return url;
    },
    renderData: function() {
        //Pre set graph colors
        var colors = ["220,220,220","151,187,205","247,70,74","70,191,189","253,180,92","148,159,177","77,83,96"];

        //Modify model URL to request for choice numbers
        var that = this;

        //Getting data for the selected field
        $.ajax({
            url: this.url(),
            type: 'GET',
            dataType: 'json',
            success: function(result) {
                that.pieData = [];
                _.each(result.data, function(data, index) {
                    var color = colors[(colors.length + index) % colors.length];
                    that.pieData.push({
                        value: data.value,
                        color: "rgba(" + color + ",1)",
                        label: (data.label)?data.label:data.htmlId
                    });
                });
                that.drawPieChart();
            }
        });
    },
    drawPieChart: function() {
        //Get charts canvas
        var pieCanvas = this.pieGraphEl.find("canvas");

        //Get Chart.js needed context
        if (_.isUndefined(this.pieCtx))
            this.pieCtx = pieCanvas.get(0).getContext("2d");

        var sum = _.reduce(this.pieData, function(memo, num) {
            return memo + num.value;
        }, 0);
        if (sum > 0) {
            //Draw charts
            if (_.isUndefined(this.pieChart))
                this.pieChart = new Chart(this.pieCtx);

            var currentChart = this.pieChart.Pie(this.pieData);
            this.pieGraphEl.find(".chart_legend").html(currentChart.generateLegend());
            this.pieGraphEl.find(".message").hide();
            pieCanvas.show();
            this.pieGraphEl.find(".chart_legend").show();
        } else {
            pieCanvas.hide();
            this.pieGraphEl.find(".chart_legend").hide();
            this.pieGraphEl.find(".message").show();
        }

    },
    updateParameters: function() {
        //Removing old parameters
        var fromDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "fromDate";
        });
        var toDate = _.find(this.parameters, function(parameter) {
            return parameter.name == "toDate";
        });
        this.parameters = _.without(this.parameters, fromDate);
        this.parameters = _.without(this.parameters, toDate);

        //Pushing new values
        if (this.model.fromDate != null) {
            this.parameters.push({name: "fromDate", values: [this.model.fromDate]});
        }
        if (this.model.toDate != null) {
            this.parameters.push({name: "toDate", values: [this.model.toDate]});
        }
        //Render graph
        this.renderData();
    }
});

// Backbone definitions - end