define([
    "jquery", "underscore", "backbone", "models/results", "backbone.validation"
], function ($, _, Backbone, ResultsModel, BackboneValidation) {
    return Backbone.View.extend({
        tagName: "fieldset",
        initialize: function () {
            this.results = new ResultsModel();
            this.render();
        },
        render: function () {
            //Render Snippet Views
            this.$el.detach();
            this.$el.empty();
            var that = this;
            if (displayTitle) {
                that.$el.append("<legend>" + formName + "</legend>");
            }
            _.each(this.collection.renderAll(), function (snippet) {
                that.$el.append(snippet);
            });
            this.$el.appendTo("#formFactoryBuild .formPlaceholder");
            this.delegateEvents({
                'click .btn[name=_eventId_next]': 'checkValidation',
                'click .btn[name=_eventId_submit]': 'checkValidation',
                'click .btn[name=_eventId_cancel]': 'submitToWebflowWithNoValidation',
                'click .btn[name=_eventId_previous]': 'submitToWebflowWithNoValidation'

            });
            BackboneValidation.bind(that,{
                model : that.results,
                valid: function (view, attr, selector) {
                    var $el = view.$('[name=' + attr + ']');
                    if($el.length>0){
                        //Simple field
                        $group = $el.closest('.control-group');
                    }else{
                        //Block of fields
                        $group = view.$("#"+attr);
                    }
                    $group.removeClass('error');
                    $group.find('.help-inline').html('').addClass('hide');
                },
                invalid: function (view, attr, error, selector) {
                    if(typeof error =='object')
                    {
                        //Block of fields error management
                        //Turn all the block in error
                        var $blockel = view.$("#"+attr).addClass("error");

                        //Browse each error to display its message
                        _.each(error, function(errorObject){
                            var $message = view.$('#'+errorObject.name+'.help-inline');
                            if(errorObject.error){
                                //Display field error
                                var $el = view.$('[name=' + errorObject.name + ']');
                                $message.html(errorObject.message);
                                $message.removeClass("hide");
                                $message.addClass("text-error");
                            }
                            else{
                                //Hide field error
                                $message.html("").addClass("hide");
                            }
                        })
                    }
                    else
                    {
                        //Simple field error management
                        var $el = view.$('[name=' + attr + ']'),
                            $group = $el.closest('.control-group');
                        $group.addClass('error');
                        $group.find('.help-inline').html(error).removeClass('hide');
                    }
                }
            });
        },
        checkValidation: function (e) {
            //Building webflow eventId basing on button id

            var webflowEvent = e.currentTarget.name;
            var data = this.$el.serializeObject();
            var jsonData = {};
            jsonData["formData"] = JSON.stringify(data);
            jsonData[webflowEvent] = webflowEvent;

            var that = this;
            //Use multipart encoding if there is a file attached to the form.
            var useMultipartEncoding = false;
            //Add all file input in data
            this.$el.find(":file").each(function(idx,elt){
                if (!_.isEmpty($(elt).val())) {
                    useMultipartEncoding = true;
                }
                data[$(elt).attr('id')] = $(elt).val();
            });
            that.results.set(data, {forceUpdate: true});
            if(useMultipartEncoding){
                $('#formFactoryTarget').attr('enctype', 'multipart/form-data');
            }
            // Check if the model is valid before saving
            // See: http://thedersen.com/projects/backbone-validation/#methods/isvalid
            if (that.results.isValid(true)) {
                Backbone.emulateJSON=true;
                //Saving data calling webflow event
                that.results.save(jsonData, { statusCode: { 200: function() {alert("Success !")} }});
            }
        },
        submitToWebflowWithNoValidation: function(e) {
            var $formFactoryTarget = $('#formFactoryTarget');
            $formFactoryTarget.prop('novalidate', true);
            $formFactoryTarget.append("<input type='hidden' name='" + e.target.name+"'/>");
            $formFactoryTarget.append("<input type='hidden' name='formData' value='" + JSON.stringify(this.$el.serializeObject()) + "'/>");
            $formFactoryTarget.append("<input type='submit' name='Submit' style='display: none' id='formFactoryTargetSubmitButton'/>");
            return $('#formFactoryTargetSubmitButton').click();
        }
    })
});
