define([
    "jquery", "underscore", "backbone"
    , "views/temp-snippet"
    , "helper/pubsub"
], function ($, _, Backbone, TempSnippetView, PubSub) {
    return Backbone.View.extend({
        tagName: "fieldset",
        initialize: function() {
            this.collection.on("add", this.render, this);
            this.collection.on("remove", this.render, this);
            this.collection.on("change", this.render, this);
            PubSub.on("mySnippetDrag", this.handleSnippetDrag, this);
            PubSub.on("tempMove", this.handleTempMove, this);
            PubSub.on("tempDrop", this.handleTempDrop, this);
            this.$formFactoryBuild = $("#formFactoryBuild");
            this.render();
        },
        render: function() {
            //Render Snippet Views
            this.$el.empty();
            var that = this;
            _.each(this.collection.renderAll(), function (snippet) {
                that.$el.append(snippet);
            });
            this.$el.appendTo("#formFactoryBuild form");
            this.delegateEvents();
        },
        handleSnippetDrag: function(mouseEvent, snippetModel) {
            $("body").append(new TempSnippetView({model: snippetModel}).render());
            this.collection.remove(snippetModel);
            PubSub.trigger("newTempPostRender", mouseEvent);
            blockInfoPopupOnSectionChange = false;
        },
        handleTempMove: function(mouseEvent) {
            var $formFactoryTarget = $('.formFactoryTarget');
            var $formFactoryTargetAfter = $('.formFactoryTargetAfter');
            $formFactoryTarget.removeClass('formFactoryTarget');
            $formFactoryTargetAfter.removeClass("formFactoryTargetAfter");
            var position = this.$formFactoryBuild.offset();
            if (mouseEvent.pageX >= position.left &&
                mouseEvent.pageX < (this.$formFactoryBuild.innerWidth() + position.left) &&
                mouseEvent.pageY >= position.top &&
                mouseEvent.pageY < (this.$formFactoryBuild.innerHeight() + position.top)) {
                // relative position of the mouse in the element
                var eventY = mouseEvent.pageY - this.$formFactoryBuild.position().top;
                // List of all elements
                var myFormBits = $(this.$el.find(".formFactoryComponent"));
                // Try to find the element the mouse is hovering over
                var topelement = _.find(myFormBits, function (renderedSnippet) {
                    return ($(renderedSnippet).position().top + ($(renderedSnippet).height() / 2)) > eventY;
                });
                if (topelement) {
                    //We are over an element
                    $(topelement).addClass("formFactoryTarget");
                } else if ($('#actions').hasClass('active')) {
                    // We are under the last action so we add a diffrent class to add the element after
                    $(_.last(myFormBits)).addClass("formFactoryTargetAfter");
                } else {
                    // Add element on top of the button
                    $(_.last(myFormBits)).addClass("formFactoryTarget");
                }

            }
        },
        handleTempDrop: function(mouseEvent, model) {
            var $formFactoryTarget = $('.formFactoryTarget');
            var $formFactoryTargetAfter = $('.formFactoryTargetAfter');
            if (mouseEvent.pageX >= this.$formFactoryBuild.offset().left &&
                mouseEvent.pageX < (this.$formFactoryBuild.width() + this.$formFactoryBuild.offset().left) &&
                mouseEvent.pageY >= this.$formFactoryBuild.offset().top &&
                mouseEvent.pageY < (this.$formFactoryBuild.height() + this.$formFactoryBuild.offset().top)) {
                var index;
                if ($formFactoryTarget.length !== 0) {
                    index = $formFactoryTarget.index();
                } else {
                    index = $formFactoryTargetAfter.index() + 1;
                }
                var collectionSize = this.collection.size();
                $formFactoryTarget.removeClass("formFactoryTarget");
                $formFactoryTargetAfter.removeClass("formFactoryTargetAfter");
                var currentButtonDEF = [];
                var currentButtonPOS = null;
                _.each(this.collection.toJSON(), function(snippet, key) {
                    if (snippet.title == "Single Button" || snippet.title == "Triple Button") {
                        currentButtonDEF = snippet;
                        currentButtonPOS = key;
                    }
                });
                this.collection.remove(this.collection.at(currentButtonPOS));
                this.collection.add(model, {at: index});
                this.collection.add(currentButtonDEF, {at: collectionSize});
                blockInfoPopupOnSectionChange = false;
            } else {
                $formFactoryTarget.removeClass("formFactoryTarget");
                $formFactoryTargetAfter.removeClass("formFactoryTargetAfter");
            }
            this.updateActionCount();
        },
        updateActionCount: function() {
            if ($("#stepActionLi").hasClass("active")) {
                var numberOfActions = _.size(this.collection.toJSON());
                var $stepActionIndicator = $("#stepActionIndicator");
                $stepActionIndicator.html(numberOfActions);
                if (numberOfActions > 0) {
                    $stepActionIndicator.removeClass("badge-important");
                    $stepActionIndicator.addClass("badge-success");
                    blockInfoPopupOnSectionChange = false;
                } else {
                    $stepActionIndicator.removeClass("badge-success");
                    $stepActionIndicator.addClass("badge-important");
                }
            }
        }
    })
});
