define([
    "jquery", "underscore", "backbone"
], function ($, _, Backbone) {
    return Backbone.View.extend({
        tagName: "div",
        className: "formFactoryComponent",
        initialize: function () {
            this.template = _.template(_snippetTemplates[this.model.idNodeType()]);
            this.popoverTemplates = {};
            var that = this;
            _.each(_popoverTemplates,function(popover, key, list) {
                that.popoverTemplates[key] = _.template(popover);
            })
        },
        render: function (withAttributes) {
            var that = this;

            // Put each validation field from the snippet model to validations object
            var validations = JSON.parse(JSON.stringify(_validationsData));
            // Delete not available validations for this snippet
            validations = this.applicableValidations(that, validations);
            if (!_.isUndefined(that.model.get("validations"))) {
                validations = this.mergeValidations(that.model.get("validations"), validations);
            }

            var content = _.template(_popoverTemplates.main)({
                "title": that.model.get("title"),
                "items": that.model.get("fields"),
                "validations": validations,
                "mode":"builder",
                "popoverTemplates": that.popoverTemplates
            });
            var values = that.model.getValues();
            values["validations"] = that.model.getValidations();
            if (withAttributes) {
                return this.$el.html(
                        that.template(values)
                    ).attr({
                        "data-content": content,
                        "data-title": that.model.get("title"),
                        "data-trigger": "manual",
                        "data-html": true,
                        "data-placement": "left"
                    }
                );
            } else {
                return this.$el.html(
                    that.template(values)
                )
            }
        },
        mergeValidations: function(modelValidations, validations) {
            for (var validationName in modelValidations) {
                for (var validationFieldName in modelValidations[validationName]["fields"]) {
                    validations[validationName]["fields"][validationFieldName]["value"] = modelValidations[validationName]["fields"][validationFieldName]["value"];
                    validations[validationName]["fields"][validationFieldName]["validationJcrId"] = modelValidations[validationName]["fields"][validationFieldName]["validationJcrId"];
                }
            }
            return validations;
        },
        applicableValidations: function(snippet, validations) {
            if (!_.isUndefined(snippet.model.get('supportedValidationTypes'))) {
                _.each(validations,function(validation,key){
                    if(_.intersection(validation.types,snippet.model.get('supportedValidationTypes')).length==0){
                        delete validations[key];
                    }
                });
            } else {
                validations = [];
            }
            return validations;
        }
    });
});